/*
 *     terraVox: an experimental voxel landscape engine
 *
 *     Copyright (C) 1999/2000 Alex J. Champandard
 *
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *     The tutorial that accompanies this source code can be found at
 *         http://www.flipcode.com
 *
 *     You can also visit the terraVox web site at
 *         http://atlas.cs.york.ac.uk/~ajc116/terraVox
 *
 *     And if you wish to contact me for any reason, I can be reached at
 *         alexjc@altavista.com
 *
 */

#ifndef __VECTOR_H
#define __VECTOR_H

#include <math.h>

class vector
{
	double v[4];

public:

	inline double &x() { return v[0]; }
	inline double &y() { return v[1]; }
	inline double &z() { return v[2]; }
	inline const double &x() const { return v[0]; }
	inline const double &y() const { return v[1]; }
	inline const double &z() const { return v[2]; }

	inline vector operator+(const vector &a) const;
	inline vector operator-(const vector &a) const;
	inline vector operator-() const;

	inline vector operator%(const vector &a) const;	// cross
	inline double operator*(const vector &a) const;	// dot

	inline vector operator*(const double d) const;
	inline vector operator/(const double d) const;

	inline vector &operator+=(const vector &a);
	inline vector &operator-=(const vector &a);
	inline vector &operator*=(const double d);
	inline vector &operator/=(const double d);

	      double &operator[](const int i)       { return v[i]; }
	const double &operator[](const int i) const { return v[i]; }

	inline bool operator==(const vector &a) const;
	inline bool operator!=(const vector &a) const;

	vector &operator=(const vector &a)
	{
		v[0] = a.v[0];
		v[1] = a.v[1];
		v[2] = a.v[2];
                v[3] = a.v[3];
		return *this;
	}
	 vector(const double X, const double Y, const double Z) { v[0]=X; v[1]=Y; v[2]=Z; v[3]=1; }
	 vector(const vector &a)
	 {
	 	v[0] = a.v[0];
	 	v[1] = a.v[1];
		v[2] = a.v[2];
                v[3] = a.v[3];
	 }
	 vector() {}
	~vector() {}
};

inline vector vector::operator+(const vector &a) const return r
{
	r[0] = v[0] + a[0];
	r[1] = v[1] + a[1];
	r[2] = v[2] + a[2];
        r[3] = 1;
}
inline vector vector::operator-(const vector &a) const return r
{
	r[0] = v[0] - a[0];
	r[1] = v[1] - a[1];
	r[2] = v[2] - a[2];
        r[3] = 1;
}
inline vector vector::operator-() const return r
{
	r[0] = -v[0];
	r[1] = -v[1];
	r[2] = -v[2];
        r[3] = 1;
}
inline vector vector::operator%(const vector &a) const return r
{
	r[0] = v[1] * a[2] - v[2] * a[1];
	r[1] = v[2] * a[0] - v[0] * a[2];
	r[2] = v[0] * a[1] - v[1] * a[0];
        r[3] = 1;
}
inline double vector::operator*(const vector &a) const
{
	return v[0]*a[0] + v[1]*a[1] + v[2]*a[2];
}
inline vector vector::operator*(const double d) const return r
{
	r[0] = v[0] * d;
	r[1] = v[1] * d;
	r[2] = v[2] * d;
        r[3] = 1;
}
inline vector vector::operator/(const double d) const return r
{
	r[0] = v[0] / d;
	r[1] = v[1] / d;
	r[2] = v[2] / d;
        r[3] = 1;
}

inline vector &vector::operator+=(const vector &a)
{
	v[0] += a[0];
	v[1] += a[1];
	v[2] += a[2];
        v[3] = 1;
	return *this;
}
inline vector &vector::operator-=(const vector &a)
{
	v[0] -= a[0];
	v[1] -= a[1];
	v[2] -= a[2];
        v[3] = 1;
	return *this;
}
inline vector &vector::operator*=(const double d)
{
	v[0] *= d;
	v[1] *= d;
	v[2] *= d;
        v[3] = 1;
	return *this;
}
inline vector &vector::operator/=(const double d)
{
        double id = 1 / d;
	v[0] *= id;
	v[1] *= id;
	v[2] *= id;
        v[3] = 1;
	return *this;
}
inline bool vector::operator==(const vector &a) const
{
	return v[0]==a[0] && v[1]==a[1] && v[2]==a[2];
}

inline bool vector::operator!=(const vector &a) const
{
	return v[0]!=a[0] || v[1]!=a[1] || v[2]!=a[2];
}

inline vector operator*(const double d, const vector &a)
{
	return a*d;
}


inline vector cross(const vector &a, const vector &b) return r
{
	r[0] = a[1] * b[2] - a[2] * b[1];
	r[1] = a[2] * b[0] - a[0] * b[2];
	r[2] = a[0] * b[1] - a[1] * b[0];
        r[3] = 1;
}
inline double dotprod(const vector &a, const vector &b)
{
	return	a[0]*b[0] + a[1]*b[1] + a[2]*b[2];
}
inline double norm(const vector &a)
{
	return sqrt(a[0]*a[0] + a[1]*a[1] + a[2]*a[2]);
}
inline vector normalize(const vector &a)
{
        double id = 1/sqrt(a[0]*a[0] + a[1]*a[1] + a[2]*a[2]);
        return vector( a[0]*id, a[1]*id, a[2]*id );
}

#endif
